@@ -21,21 +21,15 @@
 
 #include <map>
 
-#include <llvm/Constants.h>
-#include <llvm/DerivedTypes.h>
-#include <llvm/Type.h>
-
-#include "CodeGenerator_p.h"
-#include "ErrorMessage.h"
-#include "ExpressionResult_p.h"
+#include "CompilationMessage.h"
 #include "Type.h"
 #include "Type_p.h"
 #include "Value.h"
 
 #include "AST/AccessorExpression.h"
 #include "AST/BinaryExpression.h"
-#include "AST/ConstantCoumpoundExpression.h"
-#include "AST/CoumpoundExpression.h"
+#include "AST/ConstantCompoundExpression.h"
+#include "AST/CompoundExpression.h"
 #include "AST/Expression.h"
 #include "AST/FunctionDeclaration.h"
 #include "AST/Tree.h"
@@ -49,11 +43,12 @@
 #include "Function_p.h"
 #include "Utils_p.h"
 #include "VariablesManager_p.h"
-#include "ExpressionGenerationContext_p.h"
+
+#include "ASTBackend/ExpressionResult.h"
+#include "ASTBackend/GenerationVisitor.h"
 
 #include "TypesManager.h"
 #include "TypesManager_p.h"
-#include <llvm/Module.h>
 #include "ModuleData_p.h"
 
 using namespace GTLCore;
@@ -109,16 +104,16 @@ void ParserBase::parseConstantDeclaration()
     {
       getNextToken();
       AST::Expression* expression = 0 ;
-      // Check if it's a coumpound expression
+      // Check if it's a compound expression
       if( currentToken().type == GTLCore::Token::STARTBRACE )
       {
-        GTLCore::AST::Expression* coumpoundExpression = parseCoumpoundExpression( type, true );
-      /* if( coumpoundExpression->size() != *memberArraySize.begin()) // TODO: recurisvely check other sizes
+        GTLCore::AST::Expression* compoundExpression = parseCompoundExpression( type, true );
+      /* if( compoundExpression->size() != *memberArraySize.begin()) // TODO: recurisvely check other sizes
         {
-          GTL_DEBUG( memberArraySize.size() << " " << coumpoundExpression->size() );
-          reportError("Wrong coumpound initialiser size", currentToken() );
+          GTL_DEBUG( memberArraySize.size() << " " << compoundExpression->size() );
+          reportError("Wrong compound initialiser size", currentToken() );
         }*/
-        expression = coumpoundExpression;
+        expression = compoundExpression;
       } else {
         expression = parseExpression(true);
       }
@@ -130,7 +125,7 @@ void ParserBase::parseConstantDeclaration()
           GTLCore::ScopedName scopedName( nameSpace(), name );
           if( tree()->containsGlobalConstant( scopedName ) )
           {
-            reportError("Constant '" + scopedName.toString() + "' has allready been declared", currentToken());
+            reportError("Constant '" + scopedName.toString() + "' has already been declared", currentToken());
           } else {
             AST::GlobalConstantDeclaration* gcd = new AST::GlobalConstantDeclaration( scopedName, type , expression, false );
             variablesManager()->declareConstant( scopedName, gcd->variable() );
@@ -159,8 +154,8 @@ void ParserBase::parseStructDefinition()
     String name = d->currentToken.string;
     getNextToken();
     if( d->compiler->typesManager()->d->isKnownType( name ) )
-    { // It has allready been declared
-      reportError( name + " has allready been declared", d->currentToken);
+    { // It has already been declared
+      reportError( name + " has already been declared", d->currentToken);
     } else {
       std::vector<Type::StructDataMember> members;
       if( isOfType( d->currentToken, Token::STARTBRACE ) )
@@ -214,9 +209,9 @@ AST::Expression* ParserBase::parseMemberArrayConstantExpression(AST::Expression*
 {
   if( d->currentToken.type == Token::DOT or d->currentToken.type == Token::STARTBOXBRACKET )
   {
-    // Check we have a coumpound expression
-    AST::ConstantCoumpoundExpression* coumpound = dynamic_cast<AST::ConstantCoumpoundExpression*>( _expression );
-    if( not coumpound )
+    // Check we have a compound expression
+    AST::ConstantCompoundExpression* compound = dynamic_cast<AST::ConstantCompoundExpression*>( _expression );
+    if( not compound )
     {
       getNextToken();
       reportError( "Expected value of array, structure or vector type.", d->currentToken );
@@ -229,7 +224,7 @@ AST::Expression* ParserBase::parseMemberArrayConstantExpression(AST::Expression*
       getNextToken();
       if( d->currentToken.type == Token::SIZE )
       {
-        return new AST::NumberExpression<int>( coumpound->size() );
+        return new AST::NumberExpression<gtl_int32>( compound->size() );
       } else if( isOfType( d->currentToken, Token::IDENTIFIER ) ) {
         String name = d->currentToken.string;
         getNextToken();
@@ -247,12 +242,10 @@ AST::Expression* ParserBase::parseMemberArrayConstantExpression(AST::Expression*
       expr = d->compiler->convertCenter()->createConvertExpression( expr, GTLCore::Type::Integer32 );
       if( expr and expr->isConstant() )
       {
-        llvm::LLVMContext context;
-        GenerationContext gc( 0, &context, 0, 0 ,0 );
-        ExpressionGenerationContext egc(0);
-        llvm::ConstantInt* idxCST = dynamic_cast<llvm::ConstantInt*>( expr->generateValue( gc, egc).constant() );
-        GTL_ASSERT( idxCST );
-        idx = idxCST->getLimitedValue();
+        ASTBackend::GenerationVisitor visitor;
+        AST::ExpressionResultSP result = expr->generateValue(&visitor);
+        const Value& val = result.scast<ASTBackend::ExpressionResult>()->value();
+        idx = val.asInt32();
       } else {
         delete expr;
         reportError( "Expected integer constant", d->currentToken );
@@ -266,9 +259,9 @@ AST::Expression* ParserBase::parseMemberArrayConstantExpression(AST::Expression*
       GTL_ABORT("Impossible error.");
     }
     GTL_DEBUG( idx );
-    if( idx >= 0 and idx < coumpound->size() )
+    if( idx >= 0 and idx < compound->size() )
     {
-      return parseMemberArrayConstantExpression( coumpound->expressionAt( idx ) );
+      return parseMemberArrayConstantExpression( compound->expressionAt( idx ) );
     } else {
       reportError( "Invalid index.", d->currentToken );
       return 0;
@@ -315,7 +308,7 @@ AST::AccessorExpression* ParserBase::parseMemberArrayExpression(AST::AccessorExp
       } else if( _expression->type()->dataType() == Type::STRUCTURE ) {
         return parseMemberArrayExpression( new AST::StructAccessorExpression( _expression ,  structMemberNameToIdx( _expression->type(), name ) ), _constantExpression);
       } else if( _expression->type()->dataType() == Type::VECTOR ) {
-        return new AST::ArrayAccessorExpression( _expression, new AST::NumberExpression<int>( vectorMemberNameToIdx( name ) ) );
+        return new AST::ArrayAccessorExpression( _expression, new AST::NumberExpression<gtl_int32>( vectorMemberNameToIdx( name ) ) );
       } else {
         GTL_DEBUG("unexpected");
         reportUnexpected( d->currentToken );
@@ -367,13 +360,13 @@ std::list< AST::Expression* > ParserBase::parseArraySize(bool _constantExpressio
   return list_;
 }
 
-AST::Expression* ParserBase::parseCoumpoundExpression( const Type* _type, bool _constantExpression )
+AST::Expression* ParserBase::parseCompoundExpression( const Type* _type, bool _constantExpression )
 {
 //   GTL_ASSERT( _type );
   bool autoType = false;
   if( not _type )
   {
-    _type = GTLCore::TypesManager::getArray(GTLCore::Type::Float); // The real type is guessed later
+    _type = GTLCore::TypesManager::getArray(GTLCore::Type::Float32); // The real type is guessed later
     autoType = true;
   }
   GTL_DEBUG( *_type );
@@ -402,7 +395,7 @@ AST::Expression* ParserBase::parseCoumpoundExpression( const Type* _type, bool _
     if( _type->dataType() == Type::STRUCTURE and index >= _type->countStructDataMembers() )
     {
       index = _type->countStructDataMembers() - 1;
-      reportError("Invalid number of constant in coumpound expression.", currentToken());
+      reportError("Invalid number of constant in compound expression.", currentToken());
     }
     const Type* subtype = _type->d->subtypeAt( index );
     if( d->currentToken.type == Token::STARTBRACE )
@@ -418,7 +411,7 @@ AST::Expression* ParserBase::parseCoumpoundExpression( const Type* _type, bool _
           return 0;
         }
       }
-      expression = parseCoumpoundExpression( subtype, _constantExpression );
+      expression = parseCompoundExpression( subtype, _constantExpression );
     } else {
       expression = parseExpression( _constantExpression );
     }
@@ -436,7 +429,7 @@ AST::Expression* ParserBase::parseCoumpoundExpression( const Type* _type, bool _
     expression = d->compiler->convertCenter()->createConvertExpression( expression, subtype );
     if( not expression )
     {
-      reportError( "can't convert coumpound expression at " + String::number( index ) + ".", d->currentToken);
+      reportError( "can't convert compound expression at " + String::number( index ) + ".", d->currentToken);
       return 0;
     }
     expressions_.push_back( expression );
@@ -448,16 +441,16 @@ AST::Expression* ParserBase::parseCoumpoundExpression( const Type* _type, bool _
       getNextToken();
       if( _constantExpression )
       {
-        return new AST::ConstantCoumpoundExpression(_type, expressions_);
+        return new AST::ConstantCompoundExpression(_type, expressions_);
       } else {
-        return new AST::CoumpoundExpression(_type, expressions_);
+        return new AST::CompoundExpression(_type, expressions_);
       }
     }
     ++index;
   }
 }
 
-AST::Expression* ParserBase::parseTypeCoumpoundExpression(bool _constantExpression )
+AST::Expression* ParserBase::parseTypeCompoundExpression(bool _constantExpression )
 {
   const Type* type = parseType();
   if( isOfType( d->currentToken, Token::STARTBRACKET ) )
@@ -465,7 +458,7 @@ AST::Expression* ParserBase::parseTypeCoumpoundExpression(bool _constantExpressi
     AST::Expression* expression = 0;
     if( type->dataType() == Type::VECTOR or type->dataType() == Type::ARRAY or type->dataType() == Type::STRUCTURE )
     {
-      expression = parseCoumpoundExpression( type, _constantExpression);
+      expression = parseCompoundExpression( type, _constantExpression);
     } else {
       getNextToken();
       expression = parseExpression(_constantExpression, type);
@@ -516,10 +509,10 @@ AST::Expression* ParserBase::parseExpression(bool _constantExpression, const GTL
     }
   } else if( currentToken().type == Token::STARTBRACE )
   {
-    AST::Expression* coumpoundExpression = parseCoumpoundExpression( _type, true );
-    if( coumpoundExpression )
+    AST::Expression* compoundExpression = parseCompoundExpression( _type, true );
+    if( compoundExpression )
     {
-      return new AST::GlobalDataExpression( coumpoundExpression );
+      return new AST::GlobalDataExpression( compoundExpression );
     }
   } else {
     AST::Expression* expression = 0;
@@ -534,7 +527,7 @@ AST::Expression* ParserBase::parseExpression(bool _constantExpression, const GTL
         return 0;
       }
     } else if( isType(currentToken()) ) {
-      expression = parseTypeCoumpoundExpression( _constantExpression );
+      expression = parseTypeCompoundExpression( _constantExpression );
     } else {
       GTL_DEBUG("unexpected");
       reportUnexpected( d->currentToken );
@@ -603,13 +596,13 @@ AST::Expression* ParserBase::parseBinaryOperator( bool _constantExpression, AST:
       } else if( d->currentToken.isUnaryOperator() ) {
         expr = parseUnaryOperator(_constantExpression);
       } else if( currentToken().type == Token::STARTBRACE ) {
-        AST::Expression* coumpoundExpression = parseCoumpoundExpression( _lhs->type() , true );
-        if( coumpoundExpression )
+        AST::Expression* compoundExpression = parseCompoundExpression( _lhs->type() , true );
+        if( compoundExpression )
         {
-          expr = new AST::GlobalDataExpression( coumpoundExpression );
+          expr = new AST::GlobalDataExpression( compoundExpression );
         }
       } else if( isType(currentToken()) ) {
-        expr = parseTypeCoumpoundExpression(_constantExpression);      
+        expr = parseTypeCompoundExpression(_constantExpression);      
       } else {
         GTL_DEBUG("unexpected");
         reportUnexpected( d->currentToken );
@@ -914,19 +907,13 @@ void ParserBase::parseFunction()
                 Value initialiserValue;
                 if( initialiser)
                 {
-                  CodeGenerator cg( 0 );
-                  llvm::LLVMContext context;
-                  ModuleData* moduleData = new ModuleData(new llvm::Module("fake", context));
-                  GenerationContext gc( &cg, &context, 0, 0, moduleData);
-                  ExpressionGenerationContext egc(0);
-                  llvm::Constant* constant = initialiser->generateValue( gc, egc ).constant();
-                  if( constant )
-                  { // TODO don't generate a constant, do the calculation with a function in the AST
-                    initialiserValue = CodeGenerator::constantToValue(constant, ptype);
-                  } else {
+                  ASTBackend::GenerationVisitor visitor;
+                  AST::ExpressionResultSP result = initialiser->generateValue(&visitor);
+                  initialiserValue = result.scast<ASTBackend::ExpressionResult>()->value();
+                  if(!initialiserValue.isValid())
+                  {
                     reportError("Couldn't generate value for default argument", currentToken());
                   }
-                  delete moduleData;
                 }
                 params.push_back( new AST::FunctionParameter( Parameter(pname, ptype, output, varying, initialiserValue), initialiser ) );
               } else {
@@ -971,7 +958,7 @@ void ParserBase::parseFunction()
                 delete fd->function();
                 delete fd;
                 delete statement;
-                reportError( "Function '" + name.toString() + "' has allready been declared", nameToken);
+                reportError( "Function '" + name.toString() + "' has already been declared", nameToken);
               }
             }
             return;
@@ -1228,7 +1215,7 @@ AST::Statement* ParserBase::parseVariableDeclaration()
         if( d->currentToken.type == Token::EQUAL )
         {
           getNextToken(); // eat the equal
-          // Check if it's a coumpound expression
+          // Check if it's a compound expression
           initialiser = parseExpression( false, type );
           initialised = true;
           if( initialiser )
@@ -1312,7 +1299,7 @@ const Type* ParserBase::parseType()
     case Token::HALF:
     case Token::FLOAT:
       getNextToken();
-      return Type::Float;
+      return Type::Float32;
     case Token::UNSIGNED:
     {
       getNextToken();
@@ -1347,7 +1334,7 @@ AST::Expression* ParserBase::parsePrimaryExpression(bool _constantExpression)
     {
       int v = d->currentToken.i;
       getNextToken();
-      return new AST::NumberExpression<int>( v );
+      return new AST::NumberExpression<gtl_int32>( v );
     }
     case Token::FLOAT_CONSTANT:
     {
@@ -1634,7 +1621,7 @@ void ParserBase::reportError( const String& errMsg, const Token& token )
 {
   if( d->compiler)
   {
-    d->compiler->appendError( ErrorMessage( errMsg, token.line, "" ) );
+    d->compiler->appendError( CompilationMessage( CompilationMessage::ERROR, errMsg, token.line, "" ) );
   } else {
     GTL_DEBUG( errMsg);
   }
@@ -1672,7 +1659,7 @@ AST::Expression* ParserBase::createBinaryOperator( const Token& token, AST::Expr
       {
         rhs = new AST::AdditionBinaryExpression( new AST::ProxyExpression(ve), rhs );
       } else if(token.type == Token::MINUSEQUAL) {
-        rhs = new AST::SubstractionBinaryExpression( new AST::ProxyExpression(ve), rhs );
+        rhs = new AST::SubtractionBinaryExpression( new AST::ProxyExpression(ve), rhs );
       } else if(token.type == Token::MULTIPLYEQUAL) {
         rhs = new AST::MultiplicationBinaryExpression( new AST::ProxyExpression(ve), rhs );
       } else if(token.type == Token::DIVIDEEQUAL) {
@@ -1751,7 +1738,7 @@ AST::Expression* ParserBase::createBinaryOperator( const Token& token, AST::Expr
       case Token::PLUS:
         return new AST::AdditionBinaryExpression( lhs, rhs );
       case Token::MINUS:
-        return new AST::SubstractionBinaryExpression( lhs, rhs );
+        return new AST::SubtractionBinaryExpression( lhs, rhs );
       case Token::MULTIPLY:
         return new AST::MultiplicationBinaryExpression( lhs, rhs );
       case Token::DIVIDE:
@@ -1786,9 +1773,7 @@ bool ParserBase::isType( const Token& token )
 std::list<int> ParserBase::expressionsListToIntegersList( const std::list< AST::Expression* >& list )
 {
   std::list<int> integersList;
-  CodeGenerator cg( 0 );
-  llvm::LLVMContext context;
-  GenerationContext gc( &cg, &context, 0, 0, 0);
+  ASTBackend::GenerationVisitor visitor;
   for( std::list< AST::Expression* >::const_iterator it = list.begin();
        it != list.end(); ++it)
   {
@@ -1796,10 +1781,9 @@ std::list<int> ParserBase::expressionsListToIntegersList( const std::list< AST::
     {
       if( (*it)->isConstant() )
       {
-        ExpressionGenerationContext egc(0);
-        llvm::ConstantInt* v = dynamic_cast<llvm::ConstantInt* >( (*it)->generateValue( gc, egc ).constant() );
-        GTL_ASSERT( v );
-        integersList.push_back( v->getValue().getLimitedValue() );
+        AST::ExpressionResultSP result = (*it)->generateValue(&visitor);
+        const Value& val = result.scast<ASTBackend::ExpressionResult>()->value();
+        integersList.push_back( val.asInt32() );
       } else {
         reportError( "Expected constant expression.", d->currentToken );
       }
@@ -1846,7 +1830,7 @@ int ParserBase::vectorMemberNameToIdx( const GTLCore::String& _name )
   }
   GTL_DEBUG("unexpected");
   reportUnexpected( d->currentToken );
-  return -1;
+  return 0;
 }
 
 int ParserBase::structMemberNameToIdx( const GTLCore::Type* _type, const GTLCore::String& _name )
@@ -1855,6 +1839,7 @@ int ParserBase::structMemberNameToIdx( const GTLCore::Type* _type, const GTLCore
   if( index == -1 )
   {
     reportError("Unknown member: '" + _name + "' for structure " + _type->structName(), d->currentToken );
+    return 0;
   }
   return index;
 }
