@@ -19,6 +19,7 @@
 
 #include "GTLTest/Test.h"
 
+#include <GTLCore/CompilationMessages.h>
 #include "OpenShiva/Debug.h"
 
 using namespace GTLCore;
@@ -30,6 +31,7 @@ using namespace GTLCore;
 #include "TestWrapper.h"
 #include "TestKernelThreading.h"
 #include "TestKernelProgressReport.h"
+#include "TestKernelChannel.h"
 
 GTLTEST_MAIN_BEGIN(TestOpenShiva)
 GTLTEST_MAIN_ADD_CASE(TestLexer)
@@ -37,6 +39,9 @@ GTLTEST_MAIN_ADD_CASE(TestMetadataLexer )
 GTLTEST_MAIN_ADD_CASE(TestMetadataParser)
 GTLTEST_MAIN_ADD_CASE(TestSource)
 GTLTEST_MAIN_ADD_CASE(TestWrapper)
+#ifdef GTLTEST_HAVE_THREAD
 GTLTEST_MAIN_ADD_CASE(TestKernelThreading)
+#endif
 GTLTEST_MAIN_ADD_CASE(TestKernelProgressReport)
+GTLTEST_MAIN_ADD_CASE(TestKernelChannel)
 GTLTEST_MAIN_END()
